﻿using Syncfusion.UI.Xaml.Grid;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;

namespace HighlightFilterText
{
    public class CustomTextBlock : TextBlock
    {
        #region Properties

        /// <summary>
        /// Text of the Cell.
        /// </summary>
        public new string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        public new static readonly DependencyProperty TextProperty =
            DependencyProperty.Register("Text", typeof(string),
            typeof(CustomTextBlock), new FrameworkPropertyMetadata(string.Empty, FrameworkPropertyMetadataOptions.AffectsRender,
                new PropertyChangedCallback(UpdateHighlighting)));

        /// <summary>
        /// HighlightPhrase is the text which contains the text to highlight.
        /// </summary>
        public string HighlightPhrase
        {
            get { return (string)GetValue(HighlightPhraseProperty); }
            set { SetValue(HighlightPhraseProperty, value); }
        }

        public static readonly DependencyProperty HighlightPhraseProperty =
            DependencyProperty.Register("HighlightPhrase", typeof(string),
            typeof(CustomTextBlock), new FrameworkPropertyMetadata(String.Empty, FrameworkPropertyMetadataOptions.AffectsRender,
                new PropertyChangedCallback(UpdateHighlighting)));

        /// <summary>
        /// HighlightBrush is the highlighted color to the text.
        /// </summary>
        public Brush HighlightBrush
        {
            get { return (Brush)GetValue(HighlightBrushProperty); }
            set { SetValue(HighlightBrushProperty, value); }
        }

        public static readonly DependencyProperty HighlightBrushProperty =
            DependencyProperty.Register("HighlightBrush", typeof(Brush),
            typeof(CustomTextBlock), new FrameworkPropertyMetadata(Brushes.Red, FrameworkPropertyMetadataOptions.AffectsRender,
                new PropertyChangedCallback(UpdateHighlighting)));

        private static void UpdateHighlighting(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ApplyHighlight(d as CustomTextBlock);
        }

        #endregion

        #region Members

        /// <summary>
        /// Function to apply the colors to the text.
        /// </summary>
        /// <param name="tb">Customized TextBlock</param>
        private static void ApplyHighlight(CustomTextBlock tb)
        {
            string highlightPhrase = tb.HighlightPhrase;
            string text = tb.Text;

            if (String.IsNullOrEmpty(highlightPhrase))
            {
                tb.Inlines.Clear();

                tb.Inlines.Add(text);
            }

            else
            {
                int index = text.IndexOf(highlightPhrase, StringComparison.OrdinalIgnoreCase);

                tb.Inlines.Clear();

                if (index < 0) //if highlightPhrase doesn't exist in text
                    tb.Inlines.Add(text); //add text, with no background highlighting, to tb.Inlines

                else
                {
                    if (index > 0) //if highlightPhrase occurs after start of text
                        tb.Inlines.Add(text.Substring(0, index)); //add the text that exists before highlightPhrase, with no background highlighting, to tb.Inlines

                    //add the highlightPhrase, using substring to get the casing as it appears in text, with a background, to tb.Inlines
                    tb.Inlines.Add(new Run(text.Substring(index, highlightPhrase.Length))
                    {
                        Background = tb.HighlightBrush
                    });

                    index += highlightPhrase.Length; //move index to the end of the matched highlightPhrase

                    if (index < text.Length) //if the end of the matched highlightPhrase occurs before the end of text
                        tb.Inlines.Add(text.Substring(index)); //add the text that exists after highlightPhrase, with no background highlighting, to tb.Inlines
                }
            }
        }

        #endregion
    }
}
